--------------------------------------------------------------------------------
-- Copyright  2021								|
-- All Rights Reserved.								|
-- 										|
-- Permission to use, copy, modify, and distribute this software in any way	|
-- is not granted without expressed written agreement. This copyright applies	|
-- to the all code, data and artwork related to this project and all associated	|
-- projects published by the author.						|
--										|
-- More information on copyright is available from sources on-line.		|
-- Use an actual factual one!							|
--------------------------------------------------------------------------------

local AddonName, NS = ...
local L = NS.TEXTS

local PERFORMANCEBAR_MEDIUM_LATENCY = 600 -- These wer Blizz globals until they went local in 1.0 (MainMenuBarMicroButtons.lua)
local PERFORMANCEBAR_LOW_LATENCY = 300

local abs, floor = math.abs, math.floor
local clip, find, format, gsub, upper = string.trim, string.find, string.format, string.gsub, string.upper

local topoints = {
	left = "RIGHT",
	right = "LEFT",
}

local MapRects = {}
local MapTop = CreateVector2D(0,0)
local MapBottom = CreateVector2D(1,1)
local function GetPlayerMapPos()
	local mapid = C_Map.GetBestMapForUnit('player')
	if not mapid then return end
	local rects = MapRects[mapid]
	if not rects then
		rects = {}
		MapTop:OnLoad(0, 0)
		MapBottom:OnLoad(1, 1)
		local _, topleft = C_Map.GetWorldPosFromMapPos(mapid, MapTop)
		local _, bottomright = C_Map.GetWorldPosFromMapPos(mapid, MapBottom)
		if not topleft or not bottomright then return 0, 0 end
		bottomright:Subtract(topleft)
		rects = { topleft.x, topleft.y, bottomright.x, bottomright.y }
		MapRects[mapid] = rects
	end
	local x, y = UnitPosition("player")
	if not x then return end
	x = x - rects[1]
	y = y - rects[2]
	return y / rects[4], x / rects[3]
end

local function GetCursorMapPos()
	local x, y = GetCursorPosition()
	local left, top = WorldMapFrame.ScrollContainer:GetLeft(), WorldMapFrame.ScrollContainer:GetTop()
	local width = WorldMapFrame.ScrollContainer:GetWidth()
	local height = WorldMapFrame.ScrollContainer:GetHeight()
	local scale = WorldMapFrame.ScrollContainer:GetEffectiveScale()
	local cx = (x/scale - left) / width
	local cy = (top - y/scale) / height
	if cx < 0 or cx > 1 or cy < 0 or cy > 1 then
		cx, cy = nil, nil
	end
	return cx, cy
end

local function GetChatMoney(money, defaultcolor, separateThousands)
	local goldString, silverString, copperString
	local ColorGold, ColoreSilver, ColorCopper = "|cffffd100", "|cffffffff", "|cffff9430"
	if ENABLE_COLORBLIND_MODE == "1" then
		ColorGold, ColoreSilver, ColorCopper = defaultcolor, defaultcolor, defaultcolor
	end
	local gold = floor(money / (COPPER_PER_SILVER * SILVER_PER_GOLD))
	local silver = floor((money - (gold * COPPER_PER_SILVER * SILVER_PER_GOLD)) / COPPER_PER_SILVER)
	local copper = mod(money, COPPER_PER_SILVER)
	local ReturnMoney
	if gold > 0 then
		if separateThousands then
			goldString = FormatLargeNumber(gold)
		else
			goldString = gold
		end
		goldString = goldString .. ColorGold .. GOLD_AMOUNT_SYMBOL .. "|r "
	end
	silverString = not goldString and "" or " "
	if silver > 0 then
		silverString = silverString .. silver .. ColoreSilver .. SILVER_AMOUNT_SYMBOL .. "|r "
		
	end
	copperString = not (goldString and silverString) and "" or " "
	if copper > 0 then
		copperString = copperString .. copper .. ColorCopper .. COPPER_AMOUNT_SYMBOL .. "|r "
	end
--	if not goldString then
--		goldString = ""
--	end
	goldString = not goldString and "" or goldString
	return goldString .. silverString .. copperString
end

local function PortraitEventFunc(self) 
	self.Portrait3D:RefreshUnit() 
end

----------------------------- MiniButton/Tray System ----------------------------- 
local MaxTrays = 1
local TrayPool = {}
local Trays = {}
local MiniButtonSaveInfo = {}
local CurrentTray
local function GetCurrentTray()
	return CurrentTray
end
NS:AddOption("GetCurrentTray", GetCurrentTray)

--[[
local function RestMiniButtonFuncs(button)
	if button.FauxSetPoint then
		button.SetPoint = button.FauxSetPoint
		button.FauxSetPoint = nil
		button.ClearAllPoints = button.FauxClearAllPoints
		button.FauxClearAllPoints = nil
		button.SetScale = button.FauxSetScale
		button.FauxSetScale = nil
		button.SetSize = button.FauxSetSize
		button.FauxSetSize = button.FauxSetSize
		button.SetWidth = button.FauxSetWidth
		button.FauxSetWidth = nil
		button.SetHeight = button.FauxSetHeight
		button.FauxSetHeight = nil
		button.FauxParented = nil
		button.FauxUnMovable = nil
	end
end
NS:AddOption("RestMiniButtonFuncs", RestMiniButtonFuncs)
]]--
local function ResetMiniButtonPos(button)
	local Settings = MiniButtonSaveInfo[button]
	if not Settings then
		NS.DebugPrint("RestMiniButton()", "Could not find button:", button:GetName())
		return
	end
--	RestMiniButtonFuncs(button)
	button:SetParent(Settings.parent)
	button:SetScale(Settings.scale)
--	button:ClearAllPoints()
--	button:SetPoint(Settings.point, Settings.relative, Settings.topoint, Settings.x, Settings.y)
	NS.PointFrame.ClearAllPoints(button)
	NS.PointFrame.SetPoint(button, Settings.point, Settings.relative, Settings.topoint, Settings.x, Settings.y)
	MiniButtonSaveInfo[button] = nil
end
NS:AddOption("ResetMiniButtonPos", ResetMiniButtonPos)

local function RemoveTray(id)
--	if id == 1 then return end -- can't remove base tray. Probably redundant
	for i=1, #Trays do
		local f = Trays[i]
		if f:GetID() == id then
			tremove(Trays, i)
			table.wipe(f.Buttons)
			f:Hide()
			tinsert(TrayPool. f)
			break
		end
	end
end

--NS:AddOption("RemoveTray", RemoveTray)
local function SelectTray(id)
	for i=1, #Trays do
		if Trays[i]:GetID() == id then
			Trays[i]:Show()
			CurrentTray = Trays[i]
		else
			Trays[i]:Hide()
		end
	end
end
NS:AddOption("SelectTray", SelectTray)

local function SetTrayPosition(self)
	NS.PointFrame.ClearAllPoints(self)
--	NS.PointFrame.SetPoint(self, setpoint, last, topoint, x, y)
	NS.PointFrame.SetPoint(self, self.TrayPoints.p, self.TrayPoints.r, self.TrayPoints.t, self.TrayPoints.x, self.TrayPoints.y)
end

local function PopulateTray(id, settings) --Creates a new tray if required
-- for transition MiniMapTracking and GameTime buttons for Classic versions
-- see Config. NewSettings.lus and Defaults.lua
	if NS:IsValidVersion(NS.WOW_Project.WrathClassic) then
		MiniMapWorldMapButton:Hide()
	end
	local Tray
	for i=1, #Trays do
		if Trays[i]:GetID() == id then
			Tray = Trays[i]
		end
	end
	if not Tray then
		local PoolCount = #TrayPool
		if PoolCount == MaxTrays then
			L.print("No more trayas allowed!")
			return
		end
		for i=1, PoolCount do
			if TrayPool[i]:GetID() == id then
				Tray = TrayPool[i]
				tremove(TrayPool, i)
				break
			end
		end
	end
	if not Tray then
		Tray = CreateFrame("Frame", nil, NS:GetFrame("Base", 4))
		Tray:SetID(id)
		tinsert(Trays, Tray)
		Tray:SetAllPoints()
		Tray:Hide()
		Tray.Buttons = {}
	end
	Tray.Layout = Tray.Layout or function() end -- Retail (10.0.7) EditMode (or AddonCompartments) fix for reparenting Minimap Buttons to try
	table.wipe(Tray.Buttons)
	local Strata = NS:GetFrame("Utility", "MiniButton"):GetFrameStrata()
	local Level = NS:GetFrame("Utility", "MiniButton"):GetFrameLevel()
	for point, buttonslist in pairs(settings) do
		local setpoint = upper(point)
		local last, topoint = Tray, setpoint
		for i=1, #buttonslist do
			button = buttonslist[i]
			local f
			if button.stack then
				f = NS:GetFrame(button.stack, button.button)
			elseif type(button.button) == "table" then
				for k, v in ipairs(button.button) do
					if k == 1 then
						f = _G[v]
					else
						f = f[v]
					end
				end
			else
				f = _G[button.button]
			end
			if f then
				if not MiniButtonSaveInfo[f] then
					MiniButtonSaveInfo[f] = {}
					local Point, Relative, ToPoint, X, Y = f:GetPoint(1)
					MiniButtonSaveInfo[f] = { point=Point, relative=Relative, topoint=ToPoint, x=X, y=Y }
					MiniButtonSaveInfo[f].parent = f:GetParent()
					MiniButtonSaveInfo[f].scale = f:GetScale()
				end
				if not f.FauxSetPoint then
--[[
					f.FauxSetPoint = f.SetPoint
					f.SetPoint = function() end
					f.FauxClearAllPoints = f.ClearAllPoints
					f.ClearAllPoints = function() end
					f.FauxSetScale = f.SetScale
					f.SetScale = function() end
					f.FauxSetSize = f.SetSize
					f.SetSize = function() end
					f.FauxSetWidth = f.SetWidth
					f.SetWidth = function() end
					f.FauxSetHeight = f.SetHeight
					f.SetHeight = function() end
]]--
					f.FauxParented = true
					if button.system then
						f.FauxUnMovable = true
					end
				end
				if f.SetFixedFrameStrata then -- non-Classic support
					f:SetFixedFrameStrata(false)
				end
				f:SetFrameStrata(Strata)
				f:SetFrameLevel(Level)
				local x, y = button.offsetx or 0, button.offsety or 0
				tinsert(Tray.Buttons, f)
--				f:FauxClearAllPoints()
--				f:FauxSetPoint(setpoint, last, topoint, x, y)
				f:SetParent(Tray)
				if not f.TrayPoints then
					hooksecurefunc(f, "SetPoint", SetTrayPosition)
				end
				f.TrayPoints = { p=setpoint, r=last, t=topoint, x=x, y=y, }
				SetTrayPosition(f)
				topoint = topoints[point]
				if button.system then
					last = Tray.Buttons[#Tray.Buttons]
				end
				local SizeFrame = NS:GetFrame("Utility", "ButtonTray1")
				if button.scale then
--					f:FauxSetScale(button.scale)
					f:SetScale(button.scale)
				else
--					f:FauxSetSize(SizeFrame:GetHeight(), SizeFrame:GetHeight())
					f:SetSize(SizeFrame:GetHeight(), SizeFrame:GetHeight())
				end
			end
		end
	end
end
NS:AddOption("PopulateTray", PopulateTray)

----------------------------- Trinket Functions ----------------------------- 
local function UpdateTrinketCooldown(self)
	local start, duration, enable = GetInventoryItemCooldown("player", self:GetID())
--print(start, duration, enable, self:GetID())
--	if start and duration then
		self.Cooldown:StartPlay(start, duration, enable)
--	end
--	CooldownFrame_Set(self.cooldown, start, duration, enable)
end

local function GetTrinketInfo(slotid)
	local link = GetInventoryItemLink("player", slotid)
	local itemName, itemIcon
	if link then
		local _, _, id = find(link, "item:(%d+)")
		local Name, _, _, _, _, _, _, _, _, Icon = GetItemInfo(id)
		itemName, itemIcon = Name, Icon
	else
		local _, Icon = GetInventorySlotInfo("trinket"..(slotid-13).."slot")
		itemIcon = Icon
	end
	return itemIcon, itemName
end

local function UpdateTrinket(self, event, ...)
-- self.icon:SetTexture(135637) -- Classic Testing
	local icon, name = GetTrinketInfo(self:GetID())
	if not name then
-- self:SetAlpha(1) -- Classic Testing
		self:SetAlpha(0)
		return
	else
		self:SetAlpha(1)
	end
	self.icon:SetTexture(icon)
	UpdateTrinketCooldown(self)
end

local function TrinketInventoryChanged(self, event, ...)
	local unit = ...
	if unit and unit ~= "player" then
		return
	end
	UpdateTrinket(self)--, event, ...)
end

local function TrinketInit(self)
	local trinks = { "Trinket1", "Trinket2", } 
	local HideTrinkets = NS:GetSettings("UI").hidetrinkets
	for i=1, 2 do
		local Trinket = NS:GetFrame("Utility", trinks[i])
		if HideTrinkets then
			NS:UnregisterEvent("PLAYER_EQUIPMENT_CHANGED", Trinket, UpdateTrinket)
			NS:UnregisterEvent("ACTIONBAR_UPDATE_COOLDOWN", Trinket, UpdateTrinketCooldown)
			NS:UnregisterEvent("UNIT_INVENTORY_CHANGED", Trinket, TrinketInventoryChanged)
			self.Cooldown.NoPlay = true
			self:Hide()
		else
			NS:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", Trinket, UpdateTrinket)
			NS:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN", Trinket, UpdateTrinketCooldown)
			NS:RegisterEvent("UNIT_INVENTORY_CHANGED", Trinket, TrinketInventoryChanged)
			Trinket.Cooldown.NoPlay = false
			Trinket:Show()
			UpdateTrinket(Trinket)
		end
	end
end
NS:AddOption("TrinketInit", TrinketInit)

NS:RegisterForInit("PLAYER_ENTERING_WORLD", "TrinketInit", 4, TrinketInit)

local function TrinketOnLoad(self, name)
	self:RegisterForClicks("LeftButtonUp","RightButtonUp")
	self:SetAttribute("slot", self:GetID())
	self:SetAttribute("type", "item")
	self:SetAttribute("alt-slot*", ATTRIBUTE_NOOP)
	self:SetAttribute("shift-slot*", ATTRIBUTE_NOOP)
	self:SetScale(NS:GetScale())
	self.icon = self:CreateTexture("$parentIcon", "BACKGROUND", nil, 1)
	self.icon:SetAllPoints()
	self.icon:SetDesaturated(false)
	self:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	self:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
	self.Cooldown = NS:CreatePrecious(self, name)
	self.Cooldown:SetTexture("interface\\cooldown\\cooldown")
	self.Cooldown:SetFont(NS.Font, NS.FontSize + 2, "")
	if NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
		RegisterStateDriver(self, "visibility", "[petbattle] hide; show")
	end
end

local function TrinketOnEnter(self)
	GameTooltip:ClearLines()
	GameTooltip:SetOwner(self, "ANCHOR_LEFT")
	GameTooltip:SetInventoryItem("player", self:GetID())
	GameTooltip:Show()	
end

local function TrinketOnLeave(self)
	GameTooltip:Hide()
end

----------------------------- Initialise This ----------------------------- 
local Initialise = {}
Initialise.Base = {
	Cave = {
		OnLoad = function(self)
			self:SetAttribute("unit", "target")
			RegisterUnitWatch(self)
		end,
	},
	MinimapGlass = {
		OnLoad = function(self)
			self.GlassTexture = self:CreateTexture()
			self.GlassTexture:SetAllPoints()
			self.GlassTexture:SetTexture(NS.TexturePath.."MinimapOverlay")
		end,
	},
	Minimap = {
		OnLoad = function(self)
			self.PosTexture = self:CreateTexture()
			self.PosTexture:SetDrawLayer("OVERLAY", 1)
			self.PosTexture:SetSize(10, 10)
			self.PosTexture:SetTexture("Interface\\BUTTONS\\WHITE8X8")
			self.PosTexture:SetPoint("BOTTOM", UIParent)
			self.PosTexture:SetColorTexture(0, 0, 0, 0.5)
-- Added for Dragonflight
			Minimap.FMOffset = NS:GetOption("Metrics").BorderWidth
			hooksecurefunc(Minimap, "SetPoint", function(self)
				FauxMazzleFrame.ClearAllPoints(Minimap)
				FauxMazzleFrame.SetPoint(Minimap, "BOTTOM", UIParent, 0, self.FMOffset)
			end)
			Minimap:SetFrameStrata("BACKGROUND")
			Minimap:SetFrameLevel(4)

			if PlayerCastingBarFrame then
				hooksecurefunc(PlayerCastingBarFrame, "SetPoint", function()
					FauxMazzleFrame.ClearAllPoints(PlayerCastingBarFrame)
					if DUF_ClassFrame then
						FauxMazzleFrame.SetPoint(PlayerCastingBarFrame, "TOP", DUF_ClassFrame, "BOTTOM", 0, -5)
					else
						FauxMazzleFrame.SetPoint(PlayerCastingBarFrame, "TOP", UIParent, "CENTER", 0, -115)
					end
				end)
			end
		end,
	},
}

if NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
	Initialise.Base.Minimap.events = {
		"PET_BATTLE_OPENING_START",
		"PET_BATTLE_CLOSE",
	}
	Initialise.Base.Minimap.OnEvent = function(self, event)
		if not event then return end
		local PetBattleHide = NS:GetSettings("UI").petbattlehide
		local Frames = NS:GetFrame("Base")
		local Hide
		if event == "PET_BATTLE_OPENING_START" then
			Hide = true
			if PetBattleHide then 
				NS:SetViewport(true)
			end
		else
			if PetBattleHide then 
				NS:SetViewport()
			end
		end
		for k, v in pairs(Frames) do
			if PetBattleHide then
				v:SetShown(not Hide)
			else
				if (v == NS:GetFrame("Base", "Cave") or v == NS:GetFrame("Base", "MinimapGlass") or v == NS:GetFrame("Base", "Minimap")) then
					v:SetShown(not Hide)
				end
			end
		end
	end
end

Initialise.Texts = {
	[1] = {
		events = { 
			"UPDATE_INVENTORY_DURABILITY",
			"PLAYER_LOGIN", -- Also sing Entering_World event
		},
		OnEvent = function(self)  -- Armor/Durability
			local Durability, Current, Full, Percent
			local LowestCurrent, LowestFull, t1, t2, t3 = 500, 0, 0, 0, 100
			for i=1,19 do
				Current, Full = GetInventoryItemDurability(i) 
				if Current and Full then
					Percent = floor(100*Current/Full + 0.5)
					if (Percent < t3) then
						t3 = Percent
					end
					if (Current < LowestCurrent) then
						LowestCurrent = Current
						LowestFull = Full
					end
					t1 = t1 + Current
					t2 = t2 + Full
				end
			end
			if t2 == 0 then
				Durability = "N/A"
			else
				Durability = floor(t1 * 100 / t2)
			end
			local Text = L.GetText(L.Durability, Durability)
			if type(Durability) == "number" then
				if Durability > 50 then
					Text = format("|cff%2xff00", ((Durability > 50) and (255 - 2.55*Durability) or (2.55*Durability)), Durability) .. Text
				else
					Text = format("|cffff%2x00", (2.55*Durability), Durability) .. Text
				end
			else
				Text = ""
			end
			self:SetText(Text)
		end,
	},
	[2] = {
		OnLoad = function(self)
			local w, h = self:GetFont()
			local frame = NS:GetFrame("Base", "Minimap").PosTexture
			frame:SetSize(NS:GetFontMetrics(1, w, h, "99.99 99.99"))
			local BorderWidth = NS:GetOption("Metrics").BorderWidth
			BorderWidth = BorderWidth + 1
			frame:SetHeight(BorderWidth)
			self:SetDrawLayer("OVERLAY", 4)
			self.Alpha = 1
			self.PosTexture = NS:GetFrame("Base", "Minimap").PosTexture
		end,
		events = { 
			"ZONE_CHANGED_NEW_AREA",
		},
		OnEvent = function(self)  -- Position
			self.MapId = C_Map.GetBestMapForUnit("player")
		end,
		updates = 4,
		OnUpdate = function(self)
			local x, y = GetPlayerMapPos()
			local Text, Alpha = ""
			if x and y then
				x = format("%.1f", x*100)
				y = format("%.1f", y*100)
				Text = x .. " - " .. y
				Alpha = 1
			else
				Alpha = 0
			end
			self:SetText(Text)
			if self.Alpha ~= Alpha then
--				self:SetAlpha(Alpha)
				self.PosTexture:SetAlpha(Alpha)
				self.Alpha = Alpha
			end
			if not WorldMapFrame:IsShown() or not self.FizzleMapChar then return end
			self.FizzleMapChar:SetText(L.Player..":  |cffffffff"..Text.."|r")
			x, y = GetCursorMapPos()
			Text = ""
			if x and y then
				x = format("%.1f", x*100)
				y = format("%.1f", y*100)
				Text = "|cffffffff"..x .. " - " .. y.."|r"
			end
			self.FizzleMapMouse:SetText(L.Cursor..": "..Text)
		end,
	},
	[3] = {
		events = { 
			"PLAYER_ENTERING_WORLD",
			"ZONE_CHANGED",
			"ZONE_CHANGED_INDOORS",
			"ZONE_CHANGED_NEW_AREA",
		},
		OnLoad = function(self)
			self.SetTrackerCollapse = function()
				self.TrackerCollapse = NS:GetSettings("UI").trackercollapse
			end
			NS:AddOption("SetTrackerCollapse", self.SetTrackerCollapse)
		end,
		OnEvent = function(self)  -- Zone
			self:SetText(GetMinimapZoneText())
			local r, g, b = 1, 1, 1
			local ZoneType = GetZonePVPInfo()
			if ZoneType == "sanctuary" then
				r, g, b = .41, .8, .94
			elseif ZoneType == "friendly" then
				r, g, b = .1, 1, .1
			elseif ZoneType == "contested" then
				r, g, b = 1, .7, 0
			elseif ZoneType == "hostile" then
				r, g, b = 1, .1, .1
			elseif ZoneType == "arena" then
				r, g, b = 1, .1, .1
			else
				r, g, b = 1, .82, 0
			end
			self:SetTextColor(r, g, b)
			if not self.TrackerInit and self.SetTrackerCollapse then
				self.TrackerInit = true
				self:SetTrackerCollapse()
			end
			if self.TrackerCollapse then
				if WatchFrame_CollapseExpandButton_OnClick then
					if not WatchFrame.collapsed then
						securecall(WatchFrame_CollapseExpandButton_OnClick, WatchFrame)
					end
				elseif ObjectiveTracker_Collapse then
					if not ObjectiveTrackerFrame.collapsed  then
						securecall("ObjectiveTracker_Collapse")
					end
				end
----				CollapseQuestHeader(0)
			end
		end,
	},
	[4] = {
		updates = 6,
		OnUpdate = function(self)
			local _,_,Latency = GetNetStats()
			local r, g, b
--			if Latency > 500 then
			if Latency > PERFORMANCEBAR_MEDIUM_LATENCY then -- 600
				r, g, b = 1, 0, 0
--			elseif Latency > 200 then
			elseif Latency > PERFORMANCEBAR_LOW_LATENCY then -- 300
				r, g, b = 1, .82, 0
			else
				r, g, b = 0, 1, 0
			end
			self:SetText(L.GetText(L.MS, Latency))
			self:SetTextColor( r, g, b)
		end,
	},
	[5] = {
		updates = 6,
		OnUpdate = function(self)
			self:SetText(L.GetText(L.FPS, format("%0.i", GetFramerate())))
		end,
	},
	[6] = {
		updates = 5,
		OnLoad = function(self)
			self.Overlay = CreateFrame("Button", nil, self:GetParent())
			self.Overlay.Text = self
			self.Overlay:SetAllPoints(self)
			self.Overlay.Texture = self.Overlay:CreateTexture()
			self.Overlay.Texture:SetAllPoints()
			self.Overlay.Texture:SetTexture(NS.TexturePath.."ChatButton")
			self.Overlay:SetAlpha(0)
			self.Overlay:SetScript("OnEnter", function(self)
				self:SetAlpha(0.5)
			end)
			self.Overlay:SetScript("OnLeave", function(self)
				self:SetAlpha(0)
			end)
			self.Overlay:SetScript("OnMouseDown", function(self)
				self.Texture:SetTexCoord(0, 1, 1, 0)
			end)
			self.Overlay:SetScript("OnMouseUp", function(self)
				self.Texture:SetTexCoord(0, 1, 0, 1)
			end)
			self.Overlay:SetScript("OnClick", function(self)
				TimeManagerClockButton.FMShown = true
				TimeManagerClockButton:Show()
				self:Hide()
			end)
			self.Overlay:SetScript("OnHide", function(self)
				self.Text:Hide()
			end)
			self.Overlay:SetScript("OnShow", function(self)
				self.Text:Show()
			end)
			local f = CreateFrame("Button")
			f.TimeText = self.Overlay
			local function SetupClicks(self)
				if not TimeManagerClockButton then return end
				NS:UnregisterForUpdates(self, 5)
				local p = TimeManagerClockButton
				p.CloseTime = self
				self:SetParent(p)
				self:SetSize(p:GetHeight()/2, p:GetHeight()/2)
				self:SetPoint("LEFT", p, "RIGHT")
				self.Texture = self:CreateTexture()
				self.Texture:SetAllPoints()
				self.Texture:SetTexture(NS.TexturePath.."CloseTimel")
				self:SetScript("OnClick", function(self)
					self.TimeText:Show()
					self:GetParent():Hide()
					TimeManagerClockButton.FMShown = nil
				end)
				p:SetScript("OnShow", function(self)
					if not self.FMShown then
						self:Hide()
					end
				end)
				p:ClearAllPoints()
				p:SetPoint("LEFT", self.TimeText)
				p:Hide()
				p:SetParent(NS:GetFrame("Base", 4))
				local s = StopwatchFrame
				s:ClearAllPoints()
				s:SetPoint("BOTTOM", self.TimeText, "TOP")
				s:SetParent(NS:GetFrame("Base", 4))
				TimeManagerFrame:ClearAllPoints()
				TimeManagerFrame:SetPoint("BOTTOMRIGHT", NS:GetFrame("Base", 4), "TOPRIGHT", 0, 2)
				SetupClicks = nil
			end
			NS:RegisterForUpdates(f, 5, SetupClicks)
		end,
		OnUpdate = function(self)
			local Minutes, Hour, Minute
			local t = date("*t", time())
			Hour = t.hour
			Minute = t.min
			Minutes = (Hour*60) + Minute
			if (Minutes > 1440) then
				Minutes = Minutes - 1440
			end
			Minutes = abs(Minutes)
			Hour = floor(Minutes/60)
			Minute = format("%02d", mod(Minutes, 60))
			local Text = " %d:%s"
			if Minutes > 719 then
				if Minutes > 779 then
					Hour = floor(Minutes/60) - 12
				end
				Text = L.GetText(L.PM, Text)
			else
				if (Hour == 0) then
					Hour = 12
				end
				Text = L.GetText(L.AM, Text)
			end
			self:SetText(format(Text, Hour, Minute))
		end,
	},
	[7] = {
		events = {
			"PLAYER_MONEY",
			"PLAYER_LOGIN",
			"PLAYER_ENTERING_WORLD",
		},
		OnLoad = function(self)
			self.SetGoldOnly = function()
				self.GoldOnly = NS:GetSettings("UI").goldonly
				self.PrintMoney = NS:GetSettings("Chat").printmoney
				NS:PostEvent("PLAYER_MONEY")
			end
			NS:AddOption("SetGoldOnly", self.SetGoldOnly)
		end,
		OnEvent = function(self, event)
			if event == "PLAYER_LOGIN" then
				self:SetGoldOnly()
			end
			if event == "PLAYER_ENTERING_WORLD" then
				self.LastMoney = GetMoney()
				NS:UnregisterEvent(event, self)
			end
			local Money = GetMoney()
			local MoneyString = GetMoneyString(Money, true)
			if self.GoldOnly and Money > self.GoldOnly then
				local Gold = strfind(MoneyString, "t ")
				if Gold then
					MoneyString = strsub(MoneyString, 1, Gold)
				end
			end
			self:SetText(MoneyString)
			if self.PrintMoney and self.LastMoney then
				local Take = Money - self.LastMoney
				if Take ~= 0 then
--					local MoneyString = GetMoneyString(abs(Take), true)
					local MoneyString = GetChatMoney(abs(Take), "|cfffee494", true)
					local Prefix = " |cff00ff00+|r"
					if Take < 0 then
						Prefix = " |cffff0000-|r"
					end
--					|cfffef2cd
--					|cffffdd78
					MoneyString = "|cfffee494" .. MONEY_COLON .. Prefix .. MoneyString
					ChatFrame4:AddMessage(MoneyString)
					self.LastMoney = Money
				end
			end
		end,
	},
}

local function AddAnimation(self, startalpha, endalpha)
	if self.ag then return end
	self.ag = self:CreateAnimationGroup()
	self.ag:SetLooping("NONE")
	
	local anim = self.ag:CreateAnimation("Alpha")
	self.anim = anim
	anim:SetOrder(1)
	anim:SetDuration(1.4)
	self.ag.parent = self
	self.ag:SetToFinalAlpha(true)
	if endalpha > 0.5 then
		self.ag:SetScript("OnFinished", function(self)
			self.parent:SetShown(self.parent.ShowAction)
		end)
	else
		self.ag:SetScript("OnFinished", function(self)
			self.parent:SetShown(self.parent.ShowAction)
		end)
	end
end

Initialise.Utility = {
	ButtonTray1 = {
		OnLoad = function(self)
			self.Buttons = {}
			self:SetID(1)
			tinsert(Trays, self)
			SelectTray(1)
		end,
	},
	MiniButton = {
		OnLoad = function(self)
			self.Portait2D = self:CreateTexture()
			self.Portait2D:SetPoint("CENTER", 1, -1.5)
			self.Portait2D:SetSize(self:GetWidth()*0.6, self:GetHeight()*0.6)

			self.PortaitText = self:CreateTexture()
			self.PortaitText:SetPoint("CENTER", 1, -1.5)
			self.PortaitText:SetSize(self.Portait2D:GetWidth()-1, self.Portait2D:GetHeight() + 1)
			self.PortaitText:SetTexture(NS.TexturePath.."FM")
			local p3name
			if FIZZLEDEBUG then
				p3name = "FauxMazzle3D"
			end
			self.Portrait3D = CreateFrame("PlayerModel", p3name, self)
			if NS:IsValidVersion(NS.WOW_Project.Classic) then -- Might need to include TBC+ sometime
				self.Portrait3D:SetPoint("TOPLEFT", 2, -2)
				self.Portrait3D:SetPoint("BOTTOMRIGHT", -2, 2)
			else
				self.Portrait3D:SetPoint("TOPLEFT", 3, -3)
				self.Portrait3D:SetPoint("BOTTOMRIGHT", -3, 3)
			end
			self.Portrait3D:SetFrameStrata(self:GetFrameStrata())
			self.Portrait3D:SetFrameLevel(self:GetFrameLevel()+1)
			self.Overlay = CreateFrame("Button", nil, self)
			self.Overlay:SetAllPoints()
			self.Overlay:SetFrameLevel(self.Portrait3D:GetFrameLevel()+1)
			self.Overlay.Texture = self.Overlay:CreateTexture()
			self.Overlay.Texture:SetAllPoints()
			self.Overlay.Texture:SetTexture("Interface\\Minimap\\MiniMap-TrackingBorder")
			self.Overlay.Texture:SetTexCoord(0.05, 0.54180000305167, 0.03, 0.5199999833107)
			self.Overlay:SetHighlightTexture(136477)
			self.SavePortraitType = 9
			self.Overlay:SetScript("OnEnter", function(self) 
				local settings = NS:GetSettings("UI")
				self.GTTHFont, self.GTTHFontSize = GameTooltipHeaderText:GetFont()
				self.GTTTFont, self.GTTTFontSize = GameTooltipText:GetFont()
				GameTooltip:ClearLines()
				GameTooltip:SetOwner(self, "ANCHOR_LEFT")
--				GameTooltipTextSmall:SetFont(NS.Font, 9)-- ,cfg.fontFlags);
				local Size = NS.FontSize
				if NS:IsValidVersion(NS.WOW_Project.Classic, NS.WOW_Project.Burning_Crusade_Classic) then -- Might need add include WotLK+ sometime
					Size = Size + 6
				else
					Size = Size + 2
				end
				GameTooltipHeaderText:SetFont(NS.Font, Size+2, "")
				GameTooltipText:SetFont(NS.Font, Size, "")
				GameTooltip:AddLine("|cffB300CDFauxMazzle:|r")
				GameTooltip:AddLine("[".._G["EXPANSION_NAME"..GetExpansionLevel()].."]")
				GameTooltip:AddLine(L.GetText(L.LeftClick, settings.LeftButton==1 and L.Options or L.Menu).."\n"..L.GetText(L.RightClick, settings.RightButton==1 and L.Options or L.Menu))
				GameTooltip:Show()
			end)
			self.Overlay:SetScript("OnLeave", function(self) 
				GameTooltipHeaderText:SetFont(self.GTTHFont, self.GTTHFontSize, "")
				GameTooltipText:SetFont(self.GTTTFont, self.GTTTFontSize, "")
				GameTooltip:Hide()
			end)
			self.Overlay:RegisterForClicks("LeftButtonUp", "RightButtonUp")
			self.Overlay:SetScript("OnClick", function(self, button, down)
				GameTooltip:Hide()
				if InCombatLockdown() then return end
				local settings = NS:GetSettings("UI")
				if settings[button] == 1 then
					SlashCmdList[upper(AddonName)]("")
				else
					NS:GetOption("ToggleContextmenu")()
				end
			end)
			function self:GetPortraitZoom()
				return self.PortraitZoom
			end
			function self:SetPortrait(config)
				NS:UnregisterEvent("UNIT_PORTRAIT_UPDATE", self, self.EventFunc)
				self:SetScript("OnUpdate", nil)
				self.UpdateFunc = nil
				self.PortaitType = NS:GetSettings("UI").buttonicon or NS.PortaitTypes["3D"]
				if config or self.PortaitType == NS.PortaitTypes["3D"] then

					NS:SetModelPortrait(self.Portrait3D, config)
					self.EventFunc = PortraitEventFunc
--					self.Portait2D:SetTexture(nil)
--					self.Portait2D:SetVertexColor(1, 1, 1)
--					self.Portait2D:SetTexture("Interface\\BUTTONS\\WHITE8X8")
--					self.Portait2D:SetVertexColor(0.2, 0.2, 0.2)
				elseif self.PortaitType == 1 then
--					self.Portait2D:SetVertexColor(1, 1, 1)
					SetPortraitTexture(self.Portait2D, "player")
					self.EventFunc = function(self) SetPortraitTexture(self.Portait2D, "player") end
--				else
--					self.PortaitText:Show()
				end
				if self.PortaitType < 2 then
					NS:RegisterEvent("UNIT_PORTRAIT_UPDATE", self, self.EventFunc)
				end
				self.Portrait3D:Hide()
				self.Portait2D:Hide()
				self.PortaitText:Hide()
				if config or self.PortaitType == NS.PortaitTypes["3D"] then
					self.Portrait3D:Show()
				elseif self.PortaitType == NS.PortaitTypes["2D"] then
						self.Portait2D:Show()
				else
					self.PortaitText:Show()
				end
			end
		end,
	},
	Logs = {
		OnLoad = function(self)
			AddAnimation(self, 0, 1)
		end,
	},
	Meters = {
		OnLoad = function(self)
			self.meters = {}
			AddAnimation(self, 1, 0)
			function self:AddMeter(mtype, frame)
				self.meters[mtype] = frame
			end
			function self:KillMeter(mtype)
				self.meters[mtype] = nil
			end
			self:SetScript("OnShow", function(self)
				for k, v in pairs(self.meters) do
					if v and not v:IsShown() then
						v:Show()
					end
				end
			end)
		end,
	},
	Downloading = {
		OnLoad = function(self)
			self.Status = 0
			self:SetHeight(5)
			self:SetAlpha(0.8)
			self.Texture = self:CreateTexture()
			self.Texture:Hide()
			self.Texture:SetSize(self:GetSize())
			self.Texture:SetPoint("LEFT", self)
			self.Texture:SetTexture("Interface\\TargetingFrame\\UI-StatusBar")
			self.Texture:SetVertexColor(0, 1, 0)
		end,
		updates = 6,
		OnUpdate = function(self)
if FIZZLEDEBUG then
	local Status = GetFileStreamingStatus()
	if self.Status ~= Status then
    print("+MAZZLE GetFileStreamingStatus", self.Status, Status)
		if Status ~= 0 then
			self.DLState = 4
			self.Texture:Show()
		else
			self.Texture:Hide()
		end
		self.Status = Status
	end
	if Status ~= 0 then 

		if Status == 1 then -- Maybe these relate to the "playable" status??
			self.Texture:SetVertexColor(0, 1, 0)
		elseif Status == 2 then
			self.Texture:SetVertexColor(1, 1, 0)
		elseif Status == 3 then
			self.Texture:SetVertexColor(1, 0, 0)
		end

		self.DLState = self.DLState - 1
		if self.DLState == 0 then
			self.DLState = 4
		end
		local width = 0.25 * self.DLState
		self.Texture:SetWidth(self:GetWidth() * width) -- Should just cycle with/3 every second
		Status = GetBackgroundLoadingStatus() --~=0 and 1 or 0
   print("FAUXMAZZLE GetBackgroundLoadingStatus", GetBackgroundLoadingStatus()) -- in case it has more than 1 return
	end	
end -- end if FIZZLEDEBUG

--[[
			local status = GetFileStreamingStatus()
				status = (GetBackgroundLoadingStatus()~=0) and 1 or 0
			end
			if status == 0 then
				self:Hide()
			else
				self:Show()
			end
			local bandwidthIn, bandwidthOut, latencyHome, latencyWorld = GetNetStats()
			local latency = latencyHome > latencyWorld and latencyHome or latencyWorld
]]--
		end,
	},
	Trinket1 = {
		OnLoad = function(self)
			TrinketOnLoad(self, "Trinket1")
		end,
		scripts = {
			OnEnter = TrinketOnEnter,
			OnLeave = TrinketOnLeave,
		},
	},
	Trinket2 = {
		OnLoad = function(self)
			TrinketOnLoad(self, "Trinket2")
		end,
		scripts = {
			OnEnter = TrinketOnEnter,
			OnLeave = TrinketOnLeave,
		},
	},
}

Initialise.AssetsCommon = {
	[1] = { -- Because we created our own drag frame in Frames.lua for this
		OnLoad = function(self)
--			if not EssencePlayerFrame then
-- Quest tracker is movable in/post Dragonflight using Edit Mode 
			if not NS:IsValidVersion(WOW_PROJECT_MAINLINE) then
				local QuestTracker = ObjectiveTrackerFrame or QuestWatchFrame or WatchFrame -- retail, vanilla/TBC, WoTLK
-- ObjectiveTrackerFrame.HeaderMenu.MinimizeButton
				self:SetParent(QuestTracker)
				self:SetFrameLevel(6)
				self.QT = QuestTracker
				QuestTracker:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOM", 45, -5) 
				QuestTracker.ClearAllPoints = function() end
				QuestTracker.SetPoint = function() end
				QuestTracker:SetMovable(true)
				QuestTracker:SetUserPlaced(true)
				QuestTracker:SetClampedToScreen(true)
--				QuestTracker:SetHeight(800)
--				QuestTracker:SetWidth(190)
				self:SetPoint("TOPLEFT", self.QT)
				self:SetPoint("BOTTOMRIGHT", self.QT, "TOPRIGHT", 0, -15)
				self:EnableMouse(true)
				self:SetHitRectInsets(-5, -5, -5, -5)
				self:RegisterForDrag("LeftButton")
				self:SetScript("OnDragStart", function(self, button)
					if button=="LeftButton" and IsModifiedClick() then
						self.QT:StartMoving()
					end
				end)
				self:SetScript("OnDragStop", function(self, button)
					self.QT:StopMovingOrSizing()
				end)
			end


--[[
			if MicroButtonAndBagsBar then
				local MICRO_BUTTONS = {
					"CharacterMicroButton",
					"SpellbookMicroButton",
					"TalentMicroButton",
					"AchievementMicroButton",
					"QuestLogMicroButton",
					"GuildMicroButton",
					"LFDMicroButton",
					"EJMicroButton",
					"CollectionsMicroButton",
					"MainMenuMicroButton",
					"HelpMicroButton",
					"StoreMicroButton",
				}
				local BAGS = {
					"MainMenuBarBackpackButton",
					"CharacterBag0Slot",
					"CharacterBag1Slot",
					"CharacterBag2Slot",
					"CharacterBag3Slot",
				}
				if CharacterReagentBag0Slot then
					BAGS[#BAGS + 1] = "CharacterReagentBag0Slot"
				end

				local ButtonList = {}
				local function OnEnter(self)
					if MicroButtonAndBagsBar:IsMouseOver() then  -- GetMouseFocus() instead of IsMouseOver maybe???
						for l, v in pairs(ButtonList) do
							v:Show()
						end
					end
				end
				local function OnLeave(self)
					if not MicroButtonAndBagsBar:IsMouseOver() then  -- GetMouseFocus() instead of IsMouseOver maybe???
						for l, v in pairs(ButtonList) do
							v:Hide()
						end
					end
				end
				local function OnShow(self)
					if not MicroButtonAndBagsBar:IsMouseOver() then
						self:Hide()
					end
				end
				MainMenuBarBackpackButton:SetScale(0.6)
				MicroButtonAndBagsBar:ClearAllPoints() 
				MicroButtonAndBagsBar:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", 0, 300)
				MicroButtonAndBagsBar:HookScript("OnEnter", OnEnter)
				MicroButtonAndBagsBar:HookScript("OnLeave", OnLeave)
				local height = 10
]]--
--				for i=1, #MICRO_BUTTONS do
--					local f = _G[MICRO_BUTTONS[i]]
--					f:ClearAllPoints()
--					if i == 1 then
--						f:SetPoint("BOTTOMRIGHT", MicroButtonAndBagsBar, "BOTTOMRIGHT")
--					else
--						f:SetPoint("BOTTOMRIGHT", _G[MICRO_BUTTONS[i-1]], "TOPRIGHT")
--					end
--					f:HookScript("OnEnter", OnEnter)
--					f:HookScript("OnLeave", OnLeave)
--					f:HookScript("OnShow", OnShow)
--					tinsert(ButtonList, f)
--					height = height + f:GetHeight()
--
--				end
--				for i=1, #BAGS do
--					local f = _G[BAGS[i]]
--					f:ClearAllPoints()
--					if i == 1 then
--						f:SetPoint("BOTTOMRIGHT", StoreMicroButton, "TOPRIGHT", 0, 10)
--					else
--						f:SetPoint("BOTTOMRIGHT", _G[BAGS[i-1]], "TOPRIGHT")
--					end
--					f:HookScript("OnEnter", OnEnter)
--					f:HookScript("OnLeave", OnLeave)
--					f:HookScript("OnShow", OnShow)
--					tinsert(ButtonList, f)
--					height = height + f:GetHeight()
--				end
--				MicroButtonAndBagsBar:SetSize(MainMenuBarBackpackButton:GetWidth(), height)
--				BagBarExpandToggle:Hide()
--				OnLeave()
--			end
--[[
			if CharacterMicroButtonAlert then
				CharacterMicroButtonAlert.CloseButton:ClearAllPoints()
				CharacterMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if SpellbookMicroButtonAlert then
				SpellbookMicroButtonAlert.CloseButton:ClearAllPoints()
				SpellbookMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if AchievementMicroButtonAlert then
				AchievementMicroButtonAlert.CloseButton:ClearAllPoints()
				AchievementMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if QuestLogMicroButtonAlert then
				QuestLogMicroButtonAlert.CloseButton:ClearAllPoints()
				QuestLogMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if MainMenuMicroButtonAlert then
				MainMenuMicroButtonAlert.CloseButton:ClearAllPoints()
				MainMenuButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if TalentMicroButtonAlert then
				TalentMicroButtonAlert.CloseButton:ClearAllPoints()
				TalentMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if GuildMicroButtonAlert then
				GuildMicroButtonAlert.CloseButton:ClearAllPoints()
				GuildMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if LFDMicroButtonAlert then
				LFDMicroButtonAlert.CloseButton:ClearAllPoints()
				LFDMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if CollectionsMicroButtonAlert then
				CollectionsMicroButtonAlert.CloseButton:ClearAllPoints()
				CollectionsMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if EJMicroButtonAlert then
				EJMicroButtonAlert.CloseButton:ClearAllPoints()
				EJMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if StoreMicroButtonAlert then
				StoreMicroButtonAlert.CloseButton:ClearAllPoints()
				StoreMicroButtonAlert.CloseButton:SetPoint("TOPLEFT", -6, 6)
			end
			if GarrisonLandingPageMinimapButton then
				GarrisonLandingPageMinimapButton.AlertBG:ClearAllPoints()
				GarrisonLandingPageMinimapButton.AlertBG:SetPoint("BOTTOM", GarrisonLandingPageMinimapButton, "TOP")
				GarrisonLandingPageMinimapButton.AlertText:ClearAllPoints()
				GarrisonLandingPageMinimapButton.AlertText:SetPoint("BOTTOM", GarrisonLandingPageMinimapButton, "TOP")
				GarrisonLandingPageMinimapButton.AlertText:SetJustifyH("CENTER") 
			end
			if ExpansionLandingPageMinimapButton then
				ExpansionLandingPageMinimapButton.AlertBG:ClearAllPoints()
				ExpansionLandingPageMinimapButton.AlertBG:SetPoint("BOTTOM", ExpansionLandingPageMinimapButton, "TOP")
				ExpansionLandingPageMinimapButton.AlertText:ClearAllPoints()
				ExpansionLandingPageMinimapButton.AlertText:SetPoint("BOTTOM", ExpansionLandingPageMinimapButton, "TOP")
				ExpansionLandingPageMinimapButton.AlertText:SetJustifyH("CENTER") 
			end
			if NS:IsValidVersion(NS.WOW_Project.Classic) then -- Might need to include TBS+ sometime
				StreamingIcon:SetScale(0.65)			
			end
			if UIWidgetPowerBarContainerFrame then
				UIWidgetPowerBarContainerFrame:ClearAllPoints()
				UIWidgetPowerBarContainerFrame:SetPoint("TOP", 0, -20)
			end
]]--
		end,
	},
	Minimap = {
		OnLoad = function(self)
			Minimap:SetMaskTexture(NS.TexturePath .. "MinimapMask")
			if MinimapCompassTexture then
				MinimapCompassTexture:Hide()
			else
				MinimapBorder:Hide()
			end
			if MinimapNorthTag then
				MinimapNorthTag.oldShow = MinimapNorthTag.Show
				MinimapNorthTag.Show = function() end
				MinimapNorthTag:Hide()
			end
			if MinimapCluster then
				if MinimapCluster.ZoneTextButton then
					MinimapCluster.ZoneTextButton:Hide()
				end
				if MinimapCluster.BorderTop then
					MinimapCluster.BorderTop:Hide()
				end
				if MinimapCluster.InstanceDifficulty then
					NS.PointFrame.ClearAllPoints(MinimapCluster.InstanceDifficulty.Instance)
					NS.PointFrame.SetPoint(MinimapCluster.InstanceDifficulty.Instance, "TOPRIGHT", NS:GetFrame("HotSpots", 5), "BOTTOMLEFT", -5, -15)
				end
			end
			if MinimapBackdrop then
				MinimapBackdrop:Hide()
			end
			if MinimapBorderTop then
				MinimapBorderTop:Hide()
			end
			if MiniMapWorldMapButton then
				MiniMapWorldMapButton:Hide()
			end
			if MinimapZoneTextButton then
				MinimapZoneTextButton:Hide()
			end
			if MinimapZoomIn then
				MinimapZoomIn:Hide()
				MinimapZoomOut:Hide()
			end
			if MinimapToggleButton then
				MinimapToggleButton:Hide() -- red X button
			end
			if Minimap.ZoomHitArea then
				hooksecurefunc(Minimap.ZoomHitArea, "Show", function(self) self:Hide() end)
			end			
			if Minimap.ZoomIn then
				hooksecurefunc(Minimap.ZoomIn, "Show", function(self) self:Hide() end)
			end
			if Minimap.ZoomOut then
				hooksecurefunc(Minimap.ZoomOut, "Show", function(self) self:Hide() end)
			end
		end,
	},
	QuestTimerFrame = {
		OnLoad = function(self)
			QuestTimerFrame:ClearAllPoints()
			QuestTimerFrame:SetPoint("TOP", UIParent, 0, 0-(NS:GetOption("Metrics").IRefY *3))
			QuestTimerFrame.ClearAllPoints = function() end
			QuestTimerFrame.SetPoint = function() end
		end,
	},
}

function NS:InitialiseFrame(frame, key, index)
	if not Initialise[key] then
		return
	end
	local tbl = Initialise[key][index]
	if not tbl then return end
	if tbl.events then
		for i=1, #tbl.events do
			NS:RegisterEvent(tbl.events[i], frame, tbl.OnEvent)
		end
		tbl.OnEvent(frame)
	end
	if tbl.OnLoad then
		tbl.OnLoad(frame)
	end
	if tbl.updates then
		NS:RegisterForUpdates(frame, tbl.updates, tbl.OnUpdate)
	end
	if tbl.scripts then
		for k, v in pairs(tbl.scripts) do
			frame:SetScript(k, v)
		end
	end
end

local PortraitBase = {
	BloodElf2={facing=-0.57999998807907, zoom=1, pos={0, 0.05999999910593, -0.019999999776483}},
	BloodElf3={facing=-0.7, zoom=1, pos={0, 0.02, -0.01}},
	Draenei2={facing=-0.62999995708466, zoom=0.98, pos={-0.1400000013411, 0.25999999701977, -0.039999999776483}},
	Draenei3={facing=-0.66999995708466, zoom=0.98, pos={0.1599999986589, -0.030000000670552, -0.040000000745058}},
	Dwarf2={facing=-0.69000005722046, zoom=1, pos={-1.341104560737e-09, 0.1299999961257, -0.069999999329448}},
	Dwarf3={facing=-0.70000004768372, zoom=1, pos={-1.381104560737e-09, 0.059999994933605, -0.040000001192093}},
	Gnome2={facing=-0.69000005722046, zoom=1, pos={-1.381104560737e-09, 0.089999996125698, -0.079999999329448}},
	Gnome3={ facing=-1.151108, zoom=1.150000, pos={ 0.010000, 0.273332, -0.066667 }, },
	Goblin3={facing=-0.5, zoom=1, pos={0, 0.10999999940395, -0.04999999910593}},
	Human2={facing=-0.69000005722046, zoom=1, pos={-1.341104560737e-09, 0.079999996125698, -0.029999999329448}},
	Human3={ facing=-0.594446, zoom=0.980000, pos={ 0.050000, -0.000002, 0.000002 }, },
	NightElf2={facing=-0.6799999666214, zoom=1, pos={0.0099999997764826, 0.15000000566244, -0.010000000447035}},
	NightElf3={facing=-0.6599999666214, zoom=1, pos={0.0099999997764826, 0.060000005662441, -0.019999999552965}},
	Orc2={facing=-0.48000005722046, zoom=1, pos={-0.2000000013411, 0.2899999961257, 6.7055225608237e-10}},
	Orc3={facing=-0.69999998807907, zoom=1, pos={0, 0.11999999731779, -0.070000000745058}},
	Pandaren3={facing=-0.41999998569489, zoom=0.9, pos={0, 0.12000000029802, -0.060000006109476}},
	Scourge2={facing=-0.59000000596046, zoom=1, pos={0, 0.15000000074506, -0.05}},
	Scourge3={facing=-0.5900000333786, zoom=1, pos={-0.2, 0.17000000596046, -0.020000000745058}},
	Tauren2={facing=-0.77000000715256, zoom=1, pos={-2.7755575615629e-17, 0.3299999833107, -0.039999997317791}},
	Tauren3={facing=-0.71000000715256, zoom=1, pos={-0.02, 0.3699999833107, -0.079999997317791}},
	Troll2={facing=-0.79000005722046, zoom=1, pos={-1.341104560737e-09, 0.2899999961257, -0.079999999329448}},
	Troll3={facing=-0.76999997138977, zoom=0.94, pos={0.070000001490116, 0.029999999776483, -0.079999999776483}},
	Worgen2={facing=-0.64, zoom=1.1, pos={-0.03, 0.23, 0.02}},
}

if NS:IsValidVersion(NS.WOW_Project.Classic) then -- Might need to include TBC+ sometime
	local P = PortraitBase.Gnome3
	P.facing = -1.160866
	P.zoom = 1.150000
	local OS = P.pos
	OS[1] = -0.140000
	OS[2] = 0.564938
	OS[3] = -0.137162
	P = PortraitBase.Human3
	P.facing = -0.547038
	P.zoom = 0.980000
	OS = P.pos
	OS[1] = 0.050000
	OS[2] = 0.019752
	OS[3] = -0.019751

	P = PortraitBase.NightElf3
	P.facing = -0.628395
	P.zoom = 1
	OS = P.pos
	OS[1] = 0.01
	OS[2] = 0.150864
	OS[3] = -0.000246 
end

local PortraitTable = {
	BloodElf2=PortraitBase.BloodElf2,
	BloodElf3=PortraitBase.BloodElf3,
	DarkIronDwarf2=PortraitBase.Dwarf2,
	DarkIronDwarf3=PortraitBase.Dwarf3,
	Draenei2=PortraitBase.Draenei2,
	Draenei3=PortraitBase.Draenei3,

	Dracthyr2=PortraitBase.NightElf2,
	Dracthyr3=PortraitBase.NightElf3,

	Dwarf2=PortraitBase.Dwarf2,
	Dwarf3=PortraitBase.Dwarf3,
	Gnome2=PortraitBase.Gnome2,
	Gnome3=PortraitBase.Gnome3,
	Goblin2=PortraitBase.Goblin3,
	Goblin3=PortraitBase.Goblin3,
	HighmountainTauren2=PortraitBase.Tauren2,
	HighmountainTauren3=PortraitBase.Tauren3,
	Human2=PortraitBase.Human2,
	Human3=PortraitBase.Human3,
	KulTiran2=PortraitBase.Human2,
	KulTiran3=PortraitBase.Human3,
	LightforgedDraenei2=PortraitBase.Draenei2,
	LightforgedDraenei3=PortraitBase.Draenei3,
	Nightborne2=PortraitBase.BloodElf2,
	Nightborne3=PortraitBase.BloodElf3,
	NightElf2=PortraitBase.NightElf2,
	NightElf3=PortraitBase.NightElf3,
	MagharOrc2=PortraitBase.Orc2,
	MagharOrc3=PortraitBase.Orc3,
	Orc2=PortraitBase.Orc2,
	Orc3=PortraitBase.Orc3,
	Pandaren2=PortraitBase.Pandaren3,
	Pandaren3=PortraitBase.Pandaren3,
	Scourge2=PortraitBase.Scourge2,
	Scourge3=PortraitBase.Scourge3,
	Tauren2=PortraitBase.Tauren2,
	Tauren3=PortraitBase.Tauren3,
	Troll2=PortraitBase.Troll2,
	Troll3=PortraitBase.Troll3,
	VoidElf2=PortraitBase.BloodElf2,
	VoidElf3=PortraitBase.BloodElf3,
	Worgen2=PortraitBase.Worgen2,
	Worgen3=PortraitBase.Worgen2,
	ZandalariTroll2=PortraitBase.Troll2,
	ZandalariTroll3=PortraitBase.Troll3,

	Undead2=PortraitBase.Human2,
	Undead3=PortraitBase.Human3,

	Unknown2=PortraitBase.Human2,
	Unknown3=PortraitBase.Human3,
}

local Fizzlemizz = 11691

function NS:SetModelPortrait(frame, config)
	frame.PortraitZoom = 1
	frame:SetPortraitZoom(0)
	frame:SetPosition(0, 0, 0)
	frame:SetFacing(0)
	frame:SetAnimation(0)
	if not config then
		local UnitRace = gsub(UnitRace("player"), " ", "")
		local UnitSex = UnitSex("player")
		frame.PortraitSettings = UnitRace .. UnitSex
		if not PortraitTable[frame.PortraitSettings] then
			NS.DebugPrint(frame.PortraitSettings, "Model not found in FrameInitialise.lua!")
			frame.PortraitSettings = "Unknown"..UnitSex
		end
		frame:SetUnit("player")
	else
-- Set Model for Fizzle
		frame.PortraitSettings = "Gnome3"
		frame:SetDisplayInfo(Fizzlemizz)
	end
	frame:Hide()
	frame:SetFacing(PortraitTable[frame.PortraitSettings].facing)
	frame:SetPortraitZoom(PortraitTable[frame.PortraitSettings].zoom)
	frame:SetPosition(PortraitTable[frame.PortraitSettings].pos[1], PortraitTable[frame.PortraitSettings].pos[2], PortraitTable[frame.PortraitSettings].pos[3])
	frame.zoom = PortraitTable[frame.PortraitSettings].zoom
	frame:Show()
end

function NS:SendPortraitToStudio()
	local Portrait = NS:GetFrame("Utility", "MiniButton").Portrait3D
	AddonStudioLoadModel(Portrait, PortraitTable[Portrait.PortraitSettings])
end
